@@ -16,10 +16,10 @@ if [ "$(getconf LONG_BIT)" = "32" ] ; then
 fi
 
 if [ "$(getconf LONG_BIT)" = "64" ] ; then
-  if grep --silent noexec /boot/grub2/grub*.cfg ; then 
+  if grep --silent noexec {{{ grub2_boot_path }}}/grub*.cfg ; then
         sed -i "s/noexec.*//g" /etc/default/grub
         sed -i "s/noexec.*//g" /etc/grub.d/*
-        GRUBCFG=/boot/grub2/*.cfg
+        GRUBCFG={{{ grub2_boot_path }}}/*.cfg
         grub2-mkconfig -o "$GRUBCFG"
   fi
 fi
