@@ -998,7 +998,7 @@ class Group(object):
                 try:
                     group.cpe_names.add(env_yaml["product_cpes"].get_cpe_name(platform))
                 except CPEDoesNotExist:
-                    print("Unsupported platform '%s' in group '%s'." % (platform, self.id_))
+                    print("Unsupported platform '%s' in group '%s'." % (platform, group.id_))
                     raise
 
 
@@ -1007,7 +1007,7 @@ class Group(object):
             if hasattr(obj, attr) and getattr(obj, attr) is None:
                 setattr(obj, attr, getattr(self, attr))
 
-    def add_rule(self, rule):
+    def add_rule(self, rule, env_yaml=None):
         if rule is None:
             return
         if self.platforms and not rule.platforms:
@@ -1015,6 +1015,15 @@ class Group(object):
         self.rules[rule.id_] = rule
         self._pass_our_properties_on_to(rule)
 
+        # Once the rule has inherited properties, update cpe_names
+        if env_yaml:
+            for platform in rule.platforms:
+                try:
+                    rule.cpe_names.add(env_yaml["product_cpes"].get_cpe_name(platform))
+                except CPEDoesNotExist:
+                    print("Unsupported platform '%s' in rule '%s'." % (platform, rule.id_))
+                    raise
+
     def __str__(self):
         return self.id_
 
@@ -1044,6 +1053,7 @@ class Rule(object):
     }
 
     PRODUCT_REFERENCES = ("stigid", "cis",)
+    GLOBAL_REFERENCES = ("srg", "vmmsrg", "disa", "cis-csc",)
 
     def __init__(self, id_):
         self.id_ = id_
@@ -1091,6 +1101,10 @@ class Rule(object):
                    .format(fname=yaml_file, err=str(exc)))
             raise RuntimeError(msg)
 
+        # platforms are read as list from the yaml file
+        # we need them to convert to set again
+        rule.platforms = set(rule.platforms)
+
         for warning_list in rule.warnings:
             if len(warning_list) != 1:
                 raise ValueError("Only one key/value pair should exist for each dictionary")
@@ -1225,10 +1239,23 @@ class Rule(object):
                 new_items[full_label] = value
                 continue
 
+            label = full_label.split("@")[0]
+
+            # this test should occur before matching product_suffix with the product qualifier
+            # present in the reference, so it catches problems even for products that are not
+            # being built at the moment
+            if label in Rule.GLOBAL_REFERENCES:
+                msg = (
+                    "You cannot use product-qualified for the '{item_u}' reference. "
+                    "Please remove the product-qualifier and merge values with the "
+                    "existing reference if there is any. Original line: {item_q}: {value_q}"
+                    .format(item_u=label, item_q=full_label, value_q=value)
+                )
+                raise ValueError(msg)
+
             if not full_label.endswith(product_suffix):
                 continue
 
-            label = full_label.split("@")[0]
             if label in items_dict and not allow_overwrites and value != items_dict[label]:
                 msg = (
                     "There is a product-qualified '{item_q}' item, "
@@ -1529,7 +1556,7 @@ class BuildLoader(DirectoryLoader):
             if "all" not in prodtypes and self.product not in prodtypes:
                 continue
             self.all_rules.add(rule)
-            self.loaded_group.add_rule(rule)
+            self.loaded_group.add_rule(rule, env_yaml=self.env_yaml)
 
             if self.loaded_group.platforms:
                 rule.inherited_platforms += self.loaded_group.platforms
